@@ -15,11 +15,11 @@ Passwd::Keyring::PWSafe3 - Password storage based on Password Safe encrypted fil
 
 =head1 VERSION
 
-Version 0.61
+Version 0.1001
 
 =cut
 
-our $VERSION = '0.61';
+our $VERSION = '0.1001';
 
 our $APP_NAME = "Passwd::Keyring";
 our $FOLDER_NAME = "Perl-Passwd-Keyring";
@@ -27,16 +27,25 @@ our $FOLDER_NAME = "Perl-Passwd-Keyring";
 =head1 SYNOPSIS
 
 Password Safe implementation of L<Passwd::Keyring>. Passwords are
-stored in the L<Password Safe|http://passwordsafe.sourceforge.net>
+stored in the Password Safe (L<http://passwordsafe.sourceforge.net>)
 encrypted file.
 
+This module does not require Password Safe to be installed, and can be
+used as generic "store many passwords in file encrypted with single
+master password" storage. Password Safe GUI, if installed, may help
+the user to review, modify, or delete saved passwords.
+
+Note: actual handling of Password Safe format is based on L<Crypt::PWSafe3>
+module. Passwd::Keyring::PWSafe3 just wraps it into the interface compatible
+with other Passwd::Keyring backends.
+
     use Passwd::Keyring::PWSafe3;
 
     my $keyring = Passwd::Keyring::PWSafe3->new(
          app=>"blahblah scraper",
          group=>"Johnny web scrapers",
-         file=>"/home/joe/secrets.pwsafe3",        # ~/passwd-keyring.pwsafe3 by default
-         master_password=>"very secret password",  # Or callback. See below
+         file=>"/home/joe/secrets.pwsafe3",        # HOME / passwd-keyring.pwsafe3 by default
+         master_password=>"very secret password",  # Or callback. See ->new docs below
     );
 
     my $username = "John";  # or get from .ini, or from .argv...
@@ -46,7 +55,7 @@ encrypted file.
         $password = <somehow interactively prompt for password>;
 
         # securely save password for future use
-        $keyring->set_password($username, $password, "blahblah.com");
+        $keyring->set_password($username, "blahblah.com");
     }
 
     login_somewhere_using($username, $password);
@@ -54,43 +63,21 @@ encrypted file.
         $keyring->clear_password($username, "blahblah.com");
     }
 
-=head1 DESCRIPTION
-
-This module does not require Password Safe to be installed, and can be
-used as generic "store many passwords in file encrypted with single
-master password" storage. Password Safe GUI, if installed, may help
-the user to review, modify, or delete saved passwords.
-
-=over 4
-
-Official GUIs can be freely downloaded from L<the official
-site|http://passwordsafe.sourceforge.net> - both Windows and (beta)
-Linux versions are available. Apart from them there exist
-various L<compatible tools|http://passwordsafe.sourceforge.net/relatedprojects.shtml>,
-for example <Pasaffe for Gnome|https://launchpad.net/pasaffe>  or
-<PwSafe for Mac|http://passwordsafe.sourceforge.net/relatedprojects.shtml>.
-
-=back
-
-Actual handling of Password Safe format is based on L<Crypt::PWSafe3>
-module. Passwd::Keyring::PWSafe3 just wraps it into the interface
-compatible with other Passwd::Keyring backends.
-
-See L<Passwd::Keyring::Auto::KeyringAPI> for detailed comments
-on keyring methods (this document is installed with
+Note: see L<Passwd::Keyring::Auto::KeyringAPI> for detailed comments
+on keyring method semantics (this document is installed with
 C<Passwd::Keyring::Auto> package).
 
 =head1 CAVEATS
 
 Underlying module (L<Crypt::PWSafe3>) in fact rewrites the whole file
-on every save and keeps all passwords cached in memory while active.
-This means, that any attempts to use the file paralelly from a few
-programs, or from a few objects within one program, are doomed to
-cause lost updates. Also, all passwords from the file are kept in
-(unprotected) memory while keyring object is active. Therefore, it is
-recommended to use separate .psafe3 file for Passwd::Keyring::PWSafe3,
-not mixing it with normal Password Safe database, and to keep keyring
-object for a short time only, especially if modifications happen.
+on every save (with the complete password list as read on init). This
+means that any attempts to use the file paralelly from a few programs,
+or from a few objects within one program, are doomed to cause lost
+updates. Also, all passwords from the file are kept in (unprotected)
+memory while keyring object is active. Therefore, it is recommended to
+use separate .psafe3 file for Passwd::Keyring::PWSafe3, not mixing it
+with possibly used normal Password Safe database, and to delete keyring
+object after it is no longer needed.
 
 There are some limitations in L<Crypt::PWSafe3> handling of Password
 Safe format. Passwords are read and saved properly and it is possible
@@ -98,6 +85,15 @@ to alternate using them from perl, and via Password Safe GUI, but some
 less important aspects of the format, like password expiraton policy,
 may be ignored. Refer to L<Crypt::PWSafe3> docs for more details.
 
+Finally, at the moment the underlying implementation is very slow,
+depending on the CPU speed opening the ring may take from a few to
+even 15-20 seconds, saving changes is much more costly (4-8 minutes on
+my weak laptop). As L<Crypt::PWSafe3> is at it's early version, one
+may hope for some performance tuning in the future. Using C<lazy_save>
+constructor parameter reduces the cost somewhat, by avoiding costly
+save on every password change. It is also possible to construct
+passwords via Password Safe GUI and use this API only to read them.
+
 =head1 DATA MAPPING
 
 Group name is mapped to Password Safe folder.
@@ -113,54 +109,30 @@ Username and password are ... well, used as username and password.
 Initializes the processing. Croaks if Crypt::PWSafe3 is not installed or
 master password is invalid. May create password file if it is missing.
 
-Handled parameters:
+Handled named parameters: 
 
-=over 4
+- app - symbolic application name (used in password notes)
 
-=item app
+- group - name for the password group (used as folder name)
 
-Symbolic application name (used in password notes)
+- file - location of .pwsafe3 file. If not given, C<passwd-keyring.pwsafe3> in user home directory is used. Will be created if does not exist. Note: absolute path is required, relative paths are very error prone.
 
-=item group
+- master_password - password required to unlock the file. Can be
+  specified as string, or as callback returning a string (usually some
+  way of interactively asking user for the password).  The callback
+  gets two parameters: app and file.
 
-Name for the password group (used as folder name)
+  If this param is missing, module will prompt interactively for this
+  password using console prompt.
 
-=item file
+- lazy_save - if given, asks not to save the file after every change
+  (saving is fairly time consuming), but only when $keyring->save
+  is called or when keyring is destroyed.
 
-Location of .pwsafe3 file. If not given, C<passwd-keyring.pwsafe3> in
-user home directory is used. Will be created if does not exist. Note:
-absolute path is required, relative paths are very error prone.
-
-=item master_password 
-
-Password required to unlock the file. Can be given as string, or
-as callback returning a string (usually some way of interactively
-asking user for the password).  The callback gets two parameters: app
-and file.
-
-If this param is missing, module will prompt interactively for this
-password using console prompt.
-
-=item lazy_save
-
-if given, asks not to save the file after every change (saving is
-fairly time consuming), but only when $keyring->save is called or when
-keyring is destroyed.
-
-=back
-
-Note: it of course does not make much sense to keep app passwords in
-encrypted storage if master password is saved in plain text. The
-module most natural usage is to interactively ask for master password
-(and use it to protect noticeable number of application-specific
-passwords).
-
-Ideas of how to workaround this obstacle are welcome. I loosely
-consider either caching master password per desktop session
-(implementing sht. similar to ssh-agent/gpg-agent or using one of
-those somehow), or integrating the tool with PAM to use actual system
-password, or both - but while it seems doable on Linux, cross platform
-solution is not so easy.
+Note: it of course does not make much sense to keep app passwords in encrypted
+storage if master password is saved in plain text. The module most natural
+usage is to interactively ask for master password (and use it to protect
+noticeable number of application-specific passwords).
 
 =cut
 
@@ -171,7 +143,7 @@ sub new {
     $self->{app} = $args{app} || 'Passwd::Keyring::PWSafe3';
     $self->{group} = $args{group} || 'Passwd::Keyring';
     $self->{lazy_save} = $args{lazy_save};
-    my $file = $args{file} || File::Spec->catfile(File::HomeDir->users_data, "passwd-keyring.pwsafe3");
+    my $file = $args{file} || File::Spec->catfile($ENV{HOME}, "passwd-keyring.pwsafe3");
 
     unless(File::Spec->file_name_is_absolute($file)) {
         croak("Absolute path to .pwsafe3 file is required, but relative path '$file' given");
@@ -221,7 +193,7 @@ sub _prompt_for_password {
     }
 }
 
-# Find data record for given params, or undef if not found
+# Zwraca rekord dla danych parametrów. Jeśli go nie ma, zwraca undef
 sub _find_record {
     my ($self, $username, $realm) = @_;
     my $group = $self->{group};
@@ -291,7 +263,10 @@ sub clear_password {
 
     my $rec = $self->_find_record($user_name, $realm);
     if($rec) {
-        $self->{vault}->deleterecord($rec->uuid);
+        # Hooking into PwSafe3 internals to avoid artificial clearing method
+        delete $self->{vault}->{record}->{ $rec->uuid };
+        # TODO: offer fix to PwSafe3 to add delete method
+
         $self->save() unless $self->{lazy_save};
         return 1;
     } else {
@@ -331,6 +306,8 @@ sub is_persistent {
 
 Marcin Kasperski
 
+Approach inspired by L<http://www.perlmonks.org/?node_id=869620>.
+
 =head1 BUGS
 
 Please report any bugs or feature requests to 
